;;; oni-project.el --- Project configuration settings  -*- lexical-binding: t; -*-

;; Copyright (C) 2023  Tom Willemse

;; Author: Tom Willemse <tom@ryuslash.org>
;; Keywords: local
;; Version: 2024.0701.150636

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>.

;;; Commentary:

;; Configuration for Emacs' project functionality.

;;; Code:

(require 'project)

(declare-function magit-status-setup-buffer "magit-status.el")
(declare-function vterm "vterm.el")

(defun oni-project-magit ()
  "Start Magit in the current project's root directory."
  (interactive)
  (magit-status-setup-buffer (project-root (project-current t))))

(when (locate-library "magit")
  (define-key project-prefix-map (kbd "m") #'oni-project-magit)
  (add-to-list 'project-switch-commands '(oni-project-magit "Magit status")))

(defun oni-project-vterm ()
  "Start Vterm in the current project's root directory.
If a buffer already exists for running Vterm in the project's root,
switch to it. Otherwise, create a new Vterm buffer. With
\\[universal-argument] prefix arg, create a new Vterm buffer even if one
already exists."
  (interactive)
  (defvar vterm-buffer-name-string)
  (let* ((project-root (project-root (project-current t)))
         (vterm-buffer (seq-find (lambda (b)
                                   (with-current-buffer b
                                     (and (eq 'vterm-mode major-mode)
                                          (string-prefix-p project-root default-directory))))
                                 (buffer-list)))
         (default-directory project-root)
         (vterm-buffer-name-string (project-prefixed-buffer-name "vterm")))
    (if (and vterm-buffer (not current-prefix-arg))
        (pop-to-buffer vterm-buffer (bound-and-true-p display-comint-buffer-action))
      (vterm))))

(when (locate-library "vterm")
  (define-key project-prefix-map (kbd "t") #'oni-project-vterm)
  (add-to-list 'project-switch-commands '(oni-project-vterm "Terminal")))

(provide 'oni-project)
;;; oni-project.el ends here
